from plone.app.linkintegrity import testing
from plone.app.linkintegrity.parser import extractLinks
from plone.app.linkintegrity.tests.utils import set_text
from plone.app.linkintegrity.utils import getIncomingLinks
from plone.app.linkintegrity.utils import getOutgoingLinks
from plone.app.linkintegrity.utils import hasIncomingLinks
from plone.app.linkintegrity.utils import hasOutgoingLinks
from plone.app.relationfield.behavior import IRelatedItems
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import TEST_USER_NAME
from z3c.relationfield import RelationValue
from z3c.relationfield.event import _setRelation
from zc.relation.interfaces import ICatalog
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import modified

import unittest


class ReferenceGenerationTestCase(unittest.TestCase):
    """reference generation testcase"""

    layer = testing.PLONE_APP_LINKINTEGRITY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]

    def _set_related_items(self, obj, items):
        assert IRelatedItems.providedBy(obj)
        setattr(obj, "relatedItems", items)
        modified(obj)

    def _get_related_items(self, obj):
        return obj.relatedItems

    def test_is_linked(self):
        img1 = self.portal["image1"]
        doc1 = self.portal["doc1"]
        set_text(doc1, '<img src="image1"></img>')
        self.assertTrue(hasIncomingLinks(img1))

    def test_referal_to_private_files(self):
        # This tests the behaviour of the link integrity code when a to
        # be deleted item is referred to by some page the current user
        # has no permission to view. In this case the privacy of the
        # linking user should be protected, so neither the name or url
        # of the linking page should be shown. First we need to create
        # the link in question and set up the permissions accordingly.
        doc = self.portal.doc1
        img = self.portal.image1
        set_text(doc, '<a href="image1">Image 1</a>')

        roles = ("Member",)
        self.portal.manage_permission("List folder contents", roles=roles)
        self.portal.manage_permission("Delete objects", roles=roles)
        doc.manage_permission("View", roles=("Manager",), acquire=0)
        doc.manage_permission(
            "Access contents information", roles=("Manager",), acquire=0
        )

        logout()
        login(self.portal, "member")
        checkPermission = self.portal.portal_membership.checkPermission
        self.assertFalse(checkPermission("View", doc))
        self.assertFalse(checkPermission("Access contents information", doc))
        self.assertTrue(checkPermission("View", img))
        self.assertTrue(checkPermission("Access contents information", img))

        # The warning is shown.
        self.assertTrue(hasOutgoingLinks(doc))
        view = img.restrictedTraverse("delete_confirmation")
        results = view()
        self.assertIn("Potential link breakage", results)
        self.assertIn("The item is not accessible.", results)

        # delete linked item and check if the source still has the relation

        # TODO: There is a permission-problem. Deleting the relation
        # When deleting the linked obj the relation is deleted by
        # z3c.relationfield.event.breakRelations. That also fires
        # ObjectModifiedEvent on the linked obj even though the user might not
        # have the permission to edit that obj.
        # Here plone.app.versioningbehavior.subscribers.create_version_on_save
        # for the linked object is triggerted and results in
        # Unauthorized: You are not allowed to access 'save' in this context

        # self.portal.manage_delObjects(img.id)
        self.portal._delObject(img.id, suppress_events=True)

        logout()
        login(self.portal, TEST_USER_NAME)
        modified(doc)
        self.assertFalse(hasOutgoingLinks(doc))
        # doc now has a broken link and no relation :-(

    def test_link_extraction_easy(self):
        doc1 = self.portal.doc1
        set_text(doc1, '<a href="doc2">Doc 2</a>')
        self.assertEqual(extractLinks(doc1.text.raw), ("doc2",))

    def test_link_extraction_more_complex(self):
        doc2 = self.portal.doc2
        set_text(
            doc2,
            '<a href="doc1">Doc 2</a>'
            + '<a href="folder1/doc3"><img src="image1" /></a>',
        )
        self.assertEqual(
            extractLinks(doc2.text.raw), ("doc1", "folder1/doc3", "image1")
        )

    def test_broken_references(self):
        # create a temporary document to test with
        doc1a = testing.create(self.portal, "Document", id="doc1a")
        doc1 = self.portal.doc1

        self.assertEqual(len(list(getOutgoingLinks(doc1))), 0)
        set_text(doc1, '<a href="doc1a">Doc 1a</a>')
        self.assertEqual(len(list(getOutgoingLinks(doc1))), 1)
        self.assertEqual(
            [link.to_object for link in getOutgoingLinks(doc1)], [self.portal.doc1a]
        )

        # Now delete the target item, suppress events and test again,
        # The reference should be a ghost not in any folder anymore.
        # check if it has no acquisition parent!
        self.portal._delObject(doc1a.id, suppress_events=True)
        objs = [link.to_object for link in getOutgoingLinks(doc1)]
        self.assertEqual(len(objs), 1)
        obj = objs[0]
        if obj is not None:
            # Plone with fixed five.intid
            # if object is None: all fine as well.
            self.assertEqual(obj.portal_type, "Document")
            _marker = dict()
            self.assertEqual(getattr(obj, "aq_parent", _marker), _marker)

    def test_catalog_cleaned_up(self):
        doc1 = self.portal.doc1
        # Create a temporary document to test with.
        doc_temp = testing.create(self.portal, "Document", id="doc_temp")

        self.assertEqual(len(list(getIncomingLinks(doc1))), 0)
        set_text(doc_temp, '<a href="doc1">Doc 1</a>')
        self.assertEqual(len(list(getIncomingLinks(doc1))), 1)
        self.assertEqual(
            [link.from_object for link in getIncomingLinks(doc1)],
            [self.portal.doc_temp],
        )

        catalog = getUtility(ICatalog)
        rels = list(catalog.findRelations())
        self.assertEqual(len(rels), 1)

        # Now delete the source item.
        self.portal._delObject(doc_temp.id)

        # Test, if relation is removed from the relation catalog.
        catalog = getUtility(ICatalog)
        rels = list(catalog.findRelations())
        self.assertEqual(len(rels), 0)

        self.assertEqual(len(list(getIncomingLinks(doc1))), 0)

    def test_relative_upwards_link_generates_matching_reference(self):
        doc1 = self.portal.doc1
        doc3 = self.portal.folder1.doc3
        set_text(doc3, '<a href="../doc1">go!</a>')
        self.assertEqual(len(list(getOutgoingLinks(doc1))), 0)
        self.assertEqual([link.to_object for link in getOutgoingLinks(doc3)], [doc1])

    def test_unicode_links(self):
        doc1 = self.portal.doc1

        # This tests checks that hasIncomingLinks can now be used safely as it
        # eventually plays well with transaction machinery.
        # Add bad link, should not raise exception and there should not
        # be any references added.
        set_text(doc1, '<a href="ö?foo=bar&baz=bam">bug</a>')

        self.assertEqual([link for link in getOutgoingLinks(doc1)], [])

    def test_reference_orthogonality(self):
        doc = self.portal.doc1
        img = self.portal.image1
        tag = img.restrictedTraverse("@@images").tag()

        # This tests the behavior when other references already exist.
        self.assertEqual([link for link in getOutgoingLinks(doc)], [])
        self.assertEqual([link for link in getIncomingLinks(doc)], [])
        self.assertEqual([link for link in getOutgoingLinks(img)], [])
        self.assertEqual([link for link in getOutgoingLinks(img)], [])

        # Then establish a reference between the document and image as
        # a related item:
        self._set_related_items(
            doc,
            [
                img,
            ],
        )
        self.assertEqual(
            self._get_related_items(doc),
            [
                img,
            ],
        )

        # Next edit the document body and insert a link to the image,
        # which should trigger the creation of a link integrity reference:
        set_text(doc, tag)

        self.assertEqual([link.to_object for link in getOutgoingLinks(doc)], [img])

        # And the related item reference remains in place:
        self.assertEqual(
            self._get_related_items(doc),
            [
                img,
            ],
        )

        # Finally, edit the document body again, this time removing the
        # link to the image, which should trigger the removal of the
        # link integrity reference:
        set_text(doc, "where did my link go?")
        self.assertEqual([link.to_object for link in getOutgoingLinks(doc)], [])

        # And again the related item reference remains in place:
        self.assertEqual(
            self._get_related_items(doc),
            [
                img,
            ],
        )

    def test_delete_confirmation_for_any_reference(self):
        """Test, if delete confirmation shows also a warning if items are
        deleted, which are referenced by other items via a reference field.
        """
        img1 = self.portal["image1"]
        doc1 = self.portal["doc1"]

        intids_tool = getUtility(IIntIds)
        to_id = intids_tool.getId(img1)
        rel = RelationValue(to_id)
        _setRelation(doc1, "related_image", rel)

        # Test, if relation is present in the relation catalog
        catalog = getUtility(ICatalog)
        rels = list(catalog.findRelations({"to_id": to_id}))
        self.assertEqual(len(rels), 1)

        # Test, if delete_confirmation_info shows also other relations than
        # ``isReferencing``.
        info = img1.restrictedTraverse("@@delete_confirmation_info")
        breaches = info.get_breaches()
        self.assertEqual(len(breaches), 1)
        self.assertEqual(len(info.get_breaches()[0]["sources"]), 1)

    def test_sources_with_multiple_links_can_appear_multiple_times(self):
        """Test, if delete confirmation shows one document as source twice
        if it links to two of our to-be-deleted documents.
        """
        from plone.app.linkintegrity.testing import create

        doc1 = self.portal.doc1
        doc2 = self.portal.doc2
        # create a document and create links to two existing documents
        doc5 = create(self.portal, "Document", id="doc5", title="Test Page 5")
        set_text(doc5, '<a href="doc1">d1</a><a href="doc2">d2</a>')

        doc5_breaches = {r.to_object for r in getOutgoingLinks(doc5)}
        # the order of breaches is non-deterministic
        self.assertEqual({doc1, doc2}, doc5_breaches)
        view = self.portal.restrictedTraverse("@@delete_confirmation_info")
        # deleting the two referenced document results in two reported breaches
        self.assertEqual(len(view.get_breaches([doc1, doc2])), 2)

    def test_if_breaches_under_excluded_paths_are_ignored(self):
        """Test, if breaches are ignored under certain conditions."""
        folder_containing_doc4 = self.portal.folder1
        doc4 = self.portal.folder1.doc4
        doc1 = self.portal.doc1
        set_text(doc4, '<a href="doc1">d1</a>')
        # sanity check: deleting doc1 causes one breach
        doc4_breaches = {r.to_object for r in getOutgoingLinks(doc4)}
        self.assertEqual({doc1}, doc4_breaches)
        view = self.portal.restrictedTraverse("@@delete_confirmation_info")
        self.assertEqual(len(view.get_breaches([doc1])), 1)
        # now check for other ways, that should do not produce any breach:
        # a) we delete the document that links to doc1
        self.assertEqual(len(view.get_breaches([doc1, doc4])), 0)
        # b) we delete the folder, containing the document that links to doc1
        self.assertEqual(len(view.get_breaches([doc1, folder_containing_doc4])), 0)

    def test_if_same_prefix_and_breaches_work(self):
        """Verify that the same prefix does not lead to acciddental ignored
        links / breaches.
        """
        from plone.app.linkintegrity.testing import create

        doc1 = self.portal.doc1
        my_page = create(self.portal, "Document", id="my-page", title="My page")
        my_page2 = create(
            self.portal,
            "Document",
            id="my-page-being-linked",
            title="My Page Being Linked",
        )
        # Create a link, that might be overlooked since both page' ids start
        # with the same string ("my-page").
        set_text(doc1, '<a href="my-page-being-linked">.</a>')
        view = self.portal.restrictedTraverse("@@delete_confirmation_info")
        self.assertEqual(len(view.get_breaches([my_page, my_page2])), 1)



from plone.app.linkintegrity.testing import PLONE_APP_LINKINTEGRITY_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.textfield import RichTextValue
from zope.lifecycleevent import modified

import unittest


class TestCopyPaste(unittest.TestCase):

    layer = PLONE_APP_LINKINTEGRITY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_copy_paste(self):
        """Test that https://github.com/plone/Products.CMFPlone/issues/2866
        is fixed. Setting relations during copy&paste failed in Zope 4.
        """
        self.portal.invokeFactory("Image", "image", title="Image")
        self.portal.invokeFactory("Document", "document", title="Document")
        document = self.portal["document"]
        image = self.portal["image"]
        text = RichTextValue(
            '<p><img src="{portal}/resolveuid/{uid}/@@images/image/large" class="image-inline" data-linktype="image" data-scale="large" data-val="{uid}" data-mce-src="{portal}/resolveuid/{uid}/@@images/image/large" data-mce-selected="1"></p>'.format(
                portal="..", uid=image.UID()
            ),
            "text/html",
            "text/x-html-safe",
        )
        document.text = text
        modified(document)
        self.portal.invokeFactory("Folder", "folder", title="Folder")
        target = self.portal["folder"]

        copied = self.portal.manage_copyObjects("document")
        target.manage_pasteObjects(copied)
        self.assertTrue(target["document"])
        # check that linkintegrity-relations exists for both items:
        info = image.restrictedTraverse("@@delete_confirmation_info")
        breaches = info.get_breaches()
        self.assertEqual(len(breaches[0]["sources"]), 2)
        uids_objs = [i.UID() for i in [target["document"], document]]
        uids_rels = [i["uid"] for i in breaches[0]["sources"]]
        self.assertEqual(set(uids_objs), set(uids_rels))



from plone.app.linkintegrity import testing
from plone.app.linkintegrity.browser.info import DeleteConfirmationInfo
from plone.app.linkintegrity.tests.utils import set_text

import unittest


class DeleteConfirmationInfoTestCase(unittest.TestCase):
    layer = testing.PLONE_APP_LINKINTEGRITY_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

    def test_get_breaches_shape(self):
        # Just here as a sanity check.
        folder1 = self.portal.folder1
        doc1 = self.portal.doc1
        set_text(doc1, '<a href="folder1">f1</a>')
        view = DeleteConfirmationInfo(self.portal, self.request)
        self.assertEqual(
            view.get_breaches([folder1]),
            [
                {
                    "target": {
                        "uid": folder1.UID(),
                        "title": "Folder 1",
                        "url": "http://nohost/plone/folder1",
                        "portal_type": "Folder",
                        "type_title": "Folder",
                    },
                    "sources": [
                        {
                            "uid": doc1.UID(),
                            "title": "Test Page 1",
                            "url": "http://nohost/plone/doc1",
                            "accessible": 1,
                        }
                    ],
                }
            ],
        )



from Acquisition import aq_inner
from OFS.interfaces import IFolder
from plone.app.linkintegrity.utils import getIncomingLinks
from plone.app.linkintegrity.utils import linkintegrity_enabled
from plone.base import PloneMessageFactory as _
from plone.base.interfaces import IPloneSiteRoot
from plone.uuid.interfaces import IUUID
from Products.CMFCore.permissions import AccessContentsInformation
from Products.CMFCore.utils import _checkPermission
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from zope.i18n import translate

import logging
import warnings


logger = logging.getLogger(__name__)


class DeleteConfirmationInfo(BrowserView):
    template = ViewPageTemplateFile("delete_confirmation_info.pt")
    breach_count = {}

    def __init__(self, context, request):
        self.linkintegrity_enabled = linkintegrity_enabled()
        self.context = context
        self.request = request

    def __call__(self, items=None):
        if not self.linkintegrity_enabled:
            return
        if items is None:
            if IPloneSiteRoot.providedBy(self.context):
                # Checking the portal for breaches makes no sense.
                return
            else:
                items = [self.context]
        self.breaches = self.get_breaches(items)
        return self.template()

    def get_breaches(self, items=None):
        """Return breaches for multiple items.

        Breaches coming from objects in the list of items
        or their children (if a object is a folder) will be ignored.
        """
        if items is None:
            items = [self.context]
        catalog = getToolByName(self.context, "portal_catalog")
        results = []
        uids_to_ignore = set()
        self.breach_count = {}
        path2obj = dict()
        path2brains = dict()

        # build various helper structures
        for obj in items:
            obj_path = "/".join(obj.getPhysicalPath())
            path2obj[obj_path] = obj
            path2brains[obj_path] = brains_to_delete = catalog(path={"query": obj_path})
            # add the current items uid and all its children's uids to the
            # list of uids that are ignored
            uids_to_ignore.update([i.UID for i in brains_to_delete])

        excluded_paths = set(path2obj.keys())

        # determine breaches
        for obj_path, obj in path2obj.items():
            brains_to_delete = path2brains[obj_path]
            for brain_to_delete in brains_to_delete:
                try:
                    obj_to_delete = brain_to_delete.getObject()  # noqa
                except (AttributeError, KeyError):
                    logger.exception(
                        "No object found for %s! Skipping", brain_to_delete
                    )
                    continue
                # look into potential breach
                breach = self.check_object(
                    obj=obj_to_delete, excluded_paths=excluded_paths
                )
                if breach:
                    for source in breach["sources"]:
                        # Only add the breach if one the sources is not in the
                        # list of items that are to be deleted.
                        if source["uid"] not in uids_to_ignore:
                            results.append(breach)
                            break

            if IFolder.providedBy(obj):
                count = len(catalog(path={"query": obj_path}))
                count_dirs = len(catalog(path={"query": obj_path}, is_folderish=True))
                count_public = len(
                    catalog(path={"query": obj_path}, review_state="published")
                )
                if count:
                    self.breach_count[obj_path] = [count, count_dirs, count_public]

        return results

    def get_breaches_for_item(self, obj=None):
        """Get breaches for one object and its children.

        Breaches coming from the children of a folder are ignored by default.
        """
        # BBB: No direct usage is known, but keep this for backwards compatibility.
        # Sooner or later, we should use only one method.
        warnings.warn(
            """Using `get_breaches_for_item` is deprecated. Use `get_breaches`
                      instead.""",
            DeprecationWarning,
        )
        if obj is not None:
            obj = [obj]
        return self.get_breaches(obj)

    def check_object(self, obj, excluded_path=None, excluded_paths=None):
        """Check one object for breaches.
        Breaches originating from excluded_paths are ignored.
        """
        # BBB: Support old and new parameters likewise
        if excluded_paths is None:
            excluded_paths = set()
        if excluded_path:
            excluded_paths.add(excluded_path)

        breaches = {}
        direct_links = getIncomingLinks(obj, from_attribute=None)
        has_breaches = False
        for direct_link in direct_links:
            source_path = direct_link.from_path
            if not source_path:
                # link is broken
                continue
            if any(
                [
                    source_path == excluded_path
                    or source_path.startswith(excluded_path + "/")
                    for excluded_path in excluded_paths
                ]
            ):
                # source is in excluded_path
                continue
            source = direct_link.from_object
            if not breaches.get("sources"):
                breaches["sources"] = []
            breaches["sources"].append(
                {
                    "uid": IUUID(source),
                    "title": source.Title(),
                    "url": source.absolute_url(),
                    "accessible": self.is_accessible(source),
                }
            )
            has_breaches = True
        if has_breaches:
            breaches["target"] = {
                "uid": IUUID(obj),
                "title": obj.Title(),
                "url": obj.absolute_url(),
                "portal_type": obj.portal_type,
                "type_title": self.get_portal_type_title(obj),
            }
            return breaches

    def get_portal_type_title(self, obj):
        """Get the portal type title of the object."""
        context = aq_inner(self.context)
        portal_types = getToolByName(context, "portal_types")
        fti = portal_types.get(obj.portal_type)
        if fti is not None:
            type_title_msgid = fti.Title()
        else:
            type_title_msgid = obj.portal_type
        type_title = translate(type_title_msgid, context=self.request)
        return type_title

    def is_accessible(self, obj):
        return _checkPermission(AccessContentsInformation, obj)

    def objects(self):
        return [_("Objects in all"), _("Folders"), _("Published objects")]



from plone.app.linkintegrity import testing
from plone.app.linkintegrity.tests.utils import set_text
from plone.app.linkintegrity.utils import getIncomingLinks
from plone.app.linkintegrity.utils import getOutgoingLinks
from plone.app.linkintegrity.utils import hasOutgoingLinks
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.base.interfaces import IEditingSchema
from plone.registry.interfaces import IRegistry
from plone.testing.zope import Browser
from zc.relation.interfaces import ICatalog
from zope.component import getMultiAdapter
from zope.component import getUtility

import re
import transaction
import unittest


class FunctionalReferenceTestCase(unittest.TestCase):
    """functional reference testcase"""

    layer = testing.PLONE_APP_LINKINTEGRITY_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        # alsoProvides(self.request, IFormLayer)

        # Get a testbrowser
        self.browser = Browser(self.layer["app"])
        self.browser.handleErrors = False
        self.browser.addHeader("Referer", self.portal.absolute_url())
        self.browser.addHeader(
            "Authorization", f"Basic {TEST_USER_NAME:s}:{TEST_USER_PASSWORD:s}"
        )

        # Do an initial page load to make sure the bundles get compiled
        # (which currently commits a transaction)
        # before we render exception views
        self.browser.open(self.portal.absolute_url())

        setRoles(
            self.portal,
            TEST_USER_ID,
            [
                "Manager",
            ],
        )

    def _get_token(self, obj):
        return getMultiAdapter((obj, self.request), name="authenticator").token()

    @unittest.skip(
        "Re-enable after https://github.com/plone/plone.app.content/issues/38"
    )  # noqa
    def test_file_reference_linkintegrity_page_is_shown(self):
        doc1 = self.portal.doc1
        file2 = testing.create(self.portal, "File", id="file2", file=testing.GIF)

        self.assertFalse(hasOutgoingLinks(doc1))
        set_text(doc1, '<a href="file2">A File</a>')
        self.assertTrue(hasOutgoingLinks(doc1))
        self.assertIn("file2", self.portal.objectIds())

        token = self._get_token(file2)
        self.request["_authenticator"] = token

        # Make changes visible to test browser
        transaction.commit()

        self.browser.handleErrors = True
        self.browser.addHeader(
            "Authorization", f"Basic {TEST_USER_NAME:s}:{TEST_USER_PASSWORD:s}"
        )

        delete_url = "{:s}/delete_confirmation?_authenticator={:s}".format(
            file2.absolute_url(), token
        )

        # Try to remove but cancel
        self.browser.open(delete_url)

        # Validate text
        self.assertIn("Potential link breakage", self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>', self.browser.contents
        )
        self.assertIn("Would you like to delete it anyway?", self.browser.contents)

        # Click cancel button, item should stay in place
        # FIXME! This fails in Plone 6 with an internal server error,
        # but maybe no longer for the original reasons for which we skip this test.
        self.browser.getControl(name="form.buttons.Cancel").click()
        self.assertEqual(self.browser.url, file2.absolute_url() + "/view")
        self.assertIn("Removal cancelled.", self.browser.contents)
        self.assertIn("file2", self.portal.objectIds())

        # Try to remove and confirm
        self.browser.open(delete_url)
        self.browser.getControl(name="form.buttons.Delete").click()
        self.assertNotIn("file2", self.portal.objectIds())

    def test_unreferenced_removal(self):
        # This tests against #6666 and #7784, simple removal of a not
        # referenced file, which broke zeo-based installations.

        # We simply use a browser to try to delete a content item.
        self.browser.open(self.portal.doc1.absolute_url())
        self.browser.getLink("Delete").click()
        self.assertIn("Do you really want to delete this item?", self.browser.contents)
        self.browser.getControl(name="form.buttons.Delete").click()

        # The resulting page should confirm the removal:
        self.assertIn("Test Page 1 has been deleted", self.browser.contents)
        self.assertNotIn("doc1", self.portal.objectIds())

    def test_renaming_referenced_item(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests makes sure items that are linked to can still be
        # renamed (see the related bug report in #6608).  First we need
        # to create the necessary links:
        set_text(doc1, '<a href="doc2">doc2</a>')
        self.assertEqual([i.from_object for i in getIncomingLinks(doc2)], [doc1])

        # Make changes visible to testbrowseropen
        transaction.commit()

        # Then we use a browser to rename the referenced image:
        self.browser.handleErrors = True
        self.browser.open(
            "{:s}/object_rename?_authenticator={:s}".format(
                doc1.absolute_url(), self._get_token(doc1)
            )
        )

        self.browser.getControl(name="form.widgets.new_id").value = "nuname"
        self.browser.getControl(name="form.buttons.Rename").click()
        self.assertIn("Renamed 'doc1' to 'nuname'.", self.browser.contents)
        transaction.commit()

        self.assertNotIn("doc1", self.portal.objectIds())
        self.assertIn("nuname", self.portal.objectIds())
        self.assertIn(doc1, [i.from_object for i in getIncomingLinks(doc2)])

        # We simply use a browser to try to delete a content item.
        self.browser.open(doc2.absolute_url())
        self.browser.getLink("Delete").click()
        self.assertIn("Do you really want to delete this item?", self.browser.contents)
        self.assertIn("nuname", self.portal.objectIds())
        # Link breakabe page should be shown
        self.assertIn("Potential link breakage", self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/nuname">Test Page 1</a>',
            self.browser.contents,
        )
        self.browser.getControl(name="form.buttons.Delete").click()
        self.assertNotIn("doc2", self.portal.objectIds())

    def test_removal_in_subfolder(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2
        folder1 = self.portal.folder1

        # This tests ensuring link integrity when removing an referenced
        # object contained in a folder that is removed.
        set_text(doc1, '<a href="folder1/doc4">a document</a>')
        set_text(doc2, '<a href="folder1/doc4">a document</a>')

        # Make changes visible to testbrowseropen
        transaction.commit()

        # Then we try to delete the folder holding the referenced
        # document. Before we can do this we need to prevent the test
        # framework from choking on the exception we intentionally
        # throw.
        self.browser.handleErrors = True

        self.browser.open(
            "{:s}/delete_confirmation?_authenticator={:s}".format(
                folder1.absolute_url(), self._get_token(folder1)
            )
        )
        self.assertIn("Potential link breakage", self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>', self.browser.contents
        )
        self.assertIn(
            '<a href="http://nohost/plone/doc2">Test Page 2</a>', self.browser.contents
        )
        self.browser.getControl(name="form.buttons.Delete").click()
        self.assertNotIn("folder1", self.portal.objectIds())

    def test_removal_with_cookie_auth(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests ensures link integrity working correctly without
        # http basic authentication (see the bug report in #6607).
        set_text(doc1, '<a href="doc2">doc2</a>')
        transaction.commit()

        browser = Browser(self.layer["app"])
        browser.handleErrors = True
        browser.addHeader("Referer", self.portal.absolute_url())
        browser.open(f"{self.portal.absolute_url():s}/folder_contents")

        # At this point we shouldn't be able to look at the folder
        # contents (as an anonymous user):
        self.assertIn("login?came_from", browser.url)

        # So we log in via the regular plone login form and additionally check
        # that there is no 'authorization' header set afterwards:
        browser.getControl(name="__ac_name").value = TEST_USER_NAME
        browser.getControl(name="__ac_password").value = TEST_USER_PASSWORD
        browser.getControl("Log in").click()
        self.assertNotIn("authorization", [h.lower() for h in browser.headers.keys()])

        # This should lead us back to the "folder contents" listing,
        # where we try to delete the referenced document.
        browser.open(
            "{:s}/delete_confirmation?_authenticator={:s}".format(
                doc2.absolute_url(), self._get_token(doc2)
            )
        )
        self.assertIn("Potential link breakage", browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>', browser.contents
        )
        browser.getControl(name="form.buttons.Delete").click()
        self.assertNotIn("doc2", self.portal.objectIds())

    def test_linkintegrity_on_off_switch(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests switching link integrity checking on and off.
        set_text(doc1, '<a href="doc2">a document</a>')
        transaction.commit()

        # This should lead us back to the "folder contents" listing,
        # where we try to delete the referenced document. Before we can
        # do this we need to prevent the test framework from choking on
        # the exception we intentionally throw.
        self.browser.handleErrors = True

        self.browser.open(
            "{:s}/delete_confirmation?_authenticator={:s}".format(
                doc2.absolute_url(), self._get_token(doc2)
            )
        )
        self.assertIn("Potential link breakage", self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>', self.browser.contents
        )

        # Now we turn the switch for link integrity checking off via the site
        # properties and try again:
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IEditingSchema, prefix="plone")
        settings.enable_link_integrity_checks = False
        transaction.commit()
        self.browser.reload()
        self.assertNotIn("Potential link breakage", self.browser.contents)

    def test_update(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2
        doc4 = self.portal.folder1.doc4

        # This tests updating link integrity information for all site content,
        # i.e. after migrating from a previous version.
        set_text(doc1, '<a href="doc2">a document</a>')
        set_text(doc2, '<a href="folder1/doc4">a document</a>')

        catalog = getUtility(ICatalog)
        rels = [i for i in catalog.findRelations()]
        for rel in rels:
            catalog.unindex(rel)

        # Just to make sure, we check that there are no references from or to
        # these documents at this point:
        self.assertEqual([i.to_object for i in getOutgoingLinks(doc1)], [])
        self.assertEqual([i.to_object for i in getOutgoingLinks(doc2)], [])

        # An update of link integrity information for all content is triggered
        # by browsing a specific url:
        transaction.commit()
        self.browser.open(
            f"{self.portal.absolute_url():s}/updateLinkIntegrityInformation"
        )
        self.browser.getControl("Update").click()
        self.assertIn("Link integrity information updated for", self.browser.contents)

        # Now the linking documents should hold the correct link integrity
        # references:
        self.assertEqual(
            [i.to_object for i in getOutgoingLinks(doc1)],
            [
                doc2,
            ],
        )
        self.assertEqual(
            [i.to_object for i in getOutgoingLinks(doc2)],
            [
                doc4,
            ],
        )

    def test_references_on_cloned_objects(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests ensures that link integrity is correctly setup when
        # cloning an object.
        set_text(doc1, '<a href="doc2">a document</a>')

        # Next we clone the document:
        token = self._get_token(doc1)
        self.request["_authenticator"] = token
        doc1.restrictedTraverse("object_copy")()

        self.request["_authenticator"] = token
        self.portal.restrictedTraverse("object_paste")()
        self.assertIn("copy_of_doc1", self.portal)
        transaction.commit()

        # Then we try to delete the document linked by the original document
        # and its clone. Before we can do this we need to prevent the test
        # framework from choking on the exception we intentionally throw.
        self.browser.handleErrors = True

        # Now we can continue and "click" the "delete" action. The confirmation
        # page should list both documents:
        self.browser.open(
            "{:s}/delete_confirmation?_authenticator={:s}".format(
                doc2.absolute_url(), self._get_token(doc2)
            )
        )
        self.assertIn("is referenced by the following items:", self.browser.contents)
        self.assertIn("Potential link breakage", self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>', self.browser.contents
        )
        self.assertIn(
            '<a href="http://nohost/plone/copy_of_doc1"', self.browser.contents
        )

    def test_files_with_spaces_removal(self):
        doc1 = self.portal.doc1

        # This tests the behaviour when removing a referenced file that has
        # spaces in its id.  First we need to rename the existing file:
        self.portal.invokeFactory(
            "Document", id="some spaces.doc", title="A spaces doc"
        )

        self.assertIn("some spaces.doc", self.portal.objectIds())
        spaces1 = self.portal["some spaces.doc"]

        set_text(doc1, '<a href="some spaces.doc">a document</a>')

        # The document should now have a reference to the file:
        self.assertEqual([i.to_object for i in getOutgoingLinks(doc1)], [spaces1])
        transaction.commit()

        # Then we use a browser to try to delete the referenced file.
        # Before we can do this we need to prevent the test framework
        # from choking on the exception we intentionally throw.
        self.browser.handleErrors = True

        self.browser.open(
            "{:s}/delete_confirmation?_authenticator={:s}".format(
                spaces1.absolute_url(), self._get_token(spaces1)
            )
        )
        self.assertIn("Potential link breakage", self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>', self.browser.contents
        )
        self.browser.getControl(name="form.buttons.Delete").click()
        self.assertNotIn("some spaces.doc", self.portal.objectIds())

    def test_removal_via_zmi(self):
        """Delete via ZMI is no longer protedted!"""
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests ensuring link integrity when removing an object via
        # the ZMI.
        set_text(doc1, '<a href="doc2">a document</a>')
        self.assertEqual([i.to_object for i in getOutgoingLinks(doc1)], [doc2])

        transaction.commit()
        # Then we use a browser to try to delete the referenced
        # document. Before we can do this we need to prevent the test
        # framework from choking on the exception we intentionally throw.
        self.browser.handleErrors = True

        self.browser.open("http://nohost/plone/manage_main")
        self.browser.getControl(name="ids:list").getControl(value="doc2").selected = (
            True
        )

        self.browser.getControl("Delete").click()
        self.assertNotIn("doc2", self.portal.objectIds())

    def test_warn_about_content(self):
        folder1 = self.portal.folder1
        self.browser.open(
            "{:s}/delete_confirmation?_authenticator={:s}".format(
                folder1.absolute_url(), self._get_token(folder1)
            )
        )
        self.assertIn("Number of selected", self.browser.contents)
        self.assertTrue(re.search(r"2\s+Objects in all", self.browser.contents))
        self.assertTrue(re.search(r"1\s+Folders", self.browser.contents))
        self.assertTrue(re.search(r"0\s+Published objects", self.browser.contents))



from setuptools import find_packages
from setuptools import setup


version = "4.0.6"

setup(
    name="plone.app.linkintegrity",
    version=version,
    description="Manage link integrity in Plone.",
    long_description="\n\n".join(
        [
            open("README.rst").read(),
            open("CHANGES.rst").read(),
        ]
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "Framework :: Zope :: 5",
        "Intended Audience :: Other Audience",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Topic :: Internet :: WWW/HTTP :: Site Management :: Link Checking",
    ],
    keywords="link integrity plone",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://github.com/plone/plone.app.linkintegrity",
    license="GPL version 2",
    packages=find_packages(),
    namespace_packages=["plone", "plone.app"],
    include_package_data=True,
    python_requires=">=3.8",
    install_requires=[
        "setuptools",
        "plone.app.intid",
        "plone.app.relationfield",
        "plone.base",
        "plone.dexterity",
        "Products.GenericSetup",
        "Products.statusmessages",
        "plone.app.textfield",
        "plone.app.uuid",
        "plone.registry",
        "plone.uuid",
        "Zope",
        "z3c.relationfield",
        "zc.relation",
        "zope.intid",
        "zope.keyreference",
    ],
    extras_require={
        "test": [
            "plone.app.testing",
            "plone.namedfile",
            "plone.testing",
        ],
    },
    platforms="Any",
    zip_safe=False,
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
